@@ -544,6 +544,19 @@ gw_guile_procedure_to_method_public (SCM proc, SCM specializers,
   pair = scm_hashq_ref (latent_variables_hash, generic_name, SCM_BOOL_F);
 
   if (scm_is_false (pair)) {
+    SCM var = scm_hashq_ref (SCM_MODULE_OBARRAY (generics),
+                             generic_name, SCM_BOOL_F);
+    if (!scm_is_false (var)) {
+      /* some other module already forced it, hang the method */
+      gw_guile_add_subr_method (SCM_VARIABLE_REF (var),
+                                proc,
+                                specializers,
+                                scm_current_module (),
+                                scm_to_int (n_req_args), 
+                                scm_is_true (use_optional_args));
+      return;
+    }
+
     pair = scm_cons (sym_generic, SCM_EOL);
     scm_hashq_set_x (latent_variables_hash, generic_name, pair);
   } else if (!scm_is_eq (scm_car (pair), sym_generic)) {
@@ -569,6 +582,21 @@ gw_guile_procedure_to_method_public (SCM proc, SCM specializers,
 }
 #undef FUNC_NAME
 
+typedef struct {
+  GWFunctionInfo *fi;
+  void **values;
+  void *rvalue;
+} ffi_guile_call_info;
+
+typedef void* (*guile_without_func)(void*);
+
+static void*
+do_ffi_call (const ffi_guile_call_info *info)
+{
+  ffi_call (&info->fi->cif, info->fi->proc, info->rvalue, info->values);
+  return NULL;
+}
+
 static SCM
 dynproc_smob_apply (SCM smob, SCM arg_list)
 {
@@ -580,17 +608,20 @@ dynproc_smob_apply (SCM smob, SCM arg_list)
   unsigned offset;
   void *data;
   GWError error;
+  ffi_guile_call_info call_info = { fi, NULL, NULL };
   
   /* TODO: Most of this should be factored out into the core; but how
    * to deal with the arg list? */
   
   data = alloca (fi->data_area_size);
   values = (void **) data;
+  call_info.values = values;
 
   error.status = GW_ERR_NONE;
 
   offset = fi->n_req_args * sizeof (void *);
   rvalue = (void *) ((unsigned char *) data + offset);
+  call_info.rvalue = rvalue;
   offset += (fi->ret_type->type->size > sizeof(ffi_arg)
              ? fi->ret_type->type->size : sizeof(ffi_arg));
   {
@@ -614,7 +645,11 @@ dynproc_smob_apply (SCM smob, SCM arg_list)
       }
   }
   
-  ffi_call (&fi->cif, fi->proc, rvalue, values);
+  if (fi->flags & GW_FUNCTION_FLAG_LEAVE_RUNTIME)
+      scm_without_guile ((guile_without_func)do_ffi_call,
+                         &call_info);
+  else
+      do_ffi_call (&call_info);
 
   rvalue = GW_RVALUE_PTR (rvalue, fi->ret_type);
   fi->ret_type->wrap_value (&result, ARENA, &fi->ret_typespec, rvalue, &error);
